==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/content/redstone/displayLink/ClickToLinkBlockItem.java <==
package com.zurrtum.create.content.redstone.displayLink;

import com.zurrtum.create.AllDataComponents;
import com.zurrtum.create.AllSoundEvents;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.foundation.codec.CreateCodecs;
import com.zurrtum.create.infrastructure.component.ClickToLinkData;
import net.minecraft.class_124;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1747;
import net.minecraft.class_1799;
import net.minecraft.class_1838;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2680;
import net.minecraft.class_2960;
import net.minecraft.class_9279;
import net.minecraft.class_9334;

public abstract class ClickToLinkBlockItem extends class_1747 {
    public ClickToLinkBlockItem(class_2248 pBlock, class_1793 pProperties) {
        super(pBlock, pProperties);
    }

    public static boolean linkableItemAlwaysPlacesWhenUsed(class_1937 world, class_2338 pos, class_1799 stack) {
        if (stack.method_7909() instanceof ClickToLinkBlockItem blockItem) {
            return !world.method_8320(pos).method_27852(blockItem.method_7711());
        }
        return false;
    }

    @Override
    public class_1269 method_7884(class_1838 pContext) {
        class_1657 player = pContext.method_8036();
        if (player == null)
            return class_1269.field_5814;
        class_1799 stack = pContext.method_8041();
        class_2338 pos = pContext.method_8037();
        class_1937 level = pContext.method_8045();
        class_2680 state = level.method_8320(pos);
        String msgKey = getMessageTranslationKey();
        int maxDistance = getMaxDistanceFromSelection();

        if (player.method_5715() && stack.method_57826(AllDataComponents.CLICK_TO_LINK_DATA)) {
            if (level.field_9236)
                return class_1269.field_5812;
            player.method_7353(class_2561.method_43471("create." + msgKey + ".clear"), true);
            stack.method_57381(AllDataComponents.CLICK_TO_LINK_DATA);
            stack.method_57381(class_9334.field_49611);
            return class_1269.field_5812;
        }

        class_2960 placedDim = level.method_27983().method_29177();

        if (!stack.method_57826(AllDataComponents.CLICK_TO_LINK_DATA)) {
            if (!isValidTarget(level, pos)) {
                if (placeWhenInvalid()) {
                    class_1269 useOn = super.method_7884(pContext);
                    if (level.field_9236 || useOn == class_1269.field_5814)
                        return useOn;

                    class_1799 itemInHand = player.method_5998(pContext.method_20287());
                    if (!itemInHand.method_7960()) {
                        stack.method_57381(AllDataComponents.CLICK_TO_LINK_DATA);
                        stack.method_57381(class_9334.field_49611);
                    }
                    return useOn;
                }

                if (level.field_9236)
                    AllSoundEvents.DENY.playFrom(player);
                player.method_7353(class_2561.method_43471("create." + msgKey + ".invalid"), true);
                return class_1269.field_5814;
            }

            if (level.field_9236)
                return class_1269.field_5812;

            player.method_7353(class_2561.method_43471("create." + msgKey + ".set"), true);
            stack.method_57379(AllDataComponents.CLICK_TO_LINK_DATA, new ClickToLinkData(pos, placedDim));
            return class_1269.field_5812;
        }

        ClickToLinkData data = stack.method_58694(AllDataComponents.CLICK_TO_LINK_DATA);
        //noinspection DataFlowIssue
        class_2338 selectedPos = data.selectedPos();
        class_2960 selectedDim = data.selectedDim();
        class_2338 placedPos = pos.method_10079(pContext.method_8038(), state.method_45474() ? 0 : 1);

        if (maxDistance != -1 && (!selectedPos.method_19771(placedPos, maxDistance) || !selectedDim.equals(placedDim))) {
            player.method_7353(class_2561.method_43471("create." + msgKey + ".too_far").method_27692(class_124.field_1061), true);
            return class_1269.field_5814;
        }

        class_2487 beTag = new class_2487();
        beTag.method_67494("TargetOffset", class_2338.field_25064, selectedPos.method_10059(placedPos));
        beTag.method_67494("TargetDimension", class_2960.field_25139, selectedDim);
        beTag.method_67494("id", CreateCodecs.BLOCK_ENTITY_TYPE_CODEC, ((IBE<?>) method_7711()).getBlockEntityType());
        stack.method_57379(class_9334.field_49611, class_9279.method_57456(beTag));

        class_1269 useOn = super.method_7884(pContext);
        if (level.field_9236 || useOn == class_1269.field_5814)
            return useOn;

        class_1799 itemInHand = player.method_5998(pContext.method_20287());
        if (!itemInHand.method_7960()) {
            stack.method_57381(AllDataComponents.CLICK_TO_LINK_DATA);
            stack.method_57381(class_9334.field_49611);
        }
        player.method_7353(class_2561.method_43471("create." + msgKey + ".success").method_27692(class_124.field_1060), true);
        return useOn;
    }

    public abstract int getMaxDistanceFromSelection();

    public abstract String getMessageTranslationKey();

    public boolean placeWhenInvalid() {
        return false;
    }

    public boolean isValidTarget(class_1936 level, class_2338 pos) {
        return true;
    }
}

==> create-fly-1.21.10-6.0.8-5-sources.jar: com/zurrtum/create/content/redstone/displayLink/ClickToLinkBlockItem.java <==
package com.zurrtum.create.content.redstone.displayLink;

import com.zurrtum.create.AllDataComponents;
import com.zurrtum.create.AllSoundEvents;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.infrastructure.component.ClickToLinkData;
import net.minecraft.class_11580;
import net.minecraft.class_124;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1747;
import net.minecraft.class_1799;
import net.minecraft.class_1838;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2680;
import net.minecraft.class_2960;
import net.minecraft.class_9334;

public abstract class ClickToLinkBlockItem extends class_1747 {
    public ClickToLinkBlockItem(class_2248 pBlock, class_1793 pProperties) {
        super(pBlock, pProperties);
    }

    public static boolean linkableItemAlwaysPlacesWhenUsed(class_1937 world, class_2338 pos, class_1799 stack) {
        if (stack.method_7909() instanceof ClickToLinkBlockItem blockItem) {
            return !world.method_8320(pos).method_27852(blockItem.method_7711());
        }
        return false;
    }

    @Override
    public class_1269 method_7884(class_1838 pContext) {
        class_1657 player = pContext.method_8036();
        if (player == null)
            return class_1269.field_5814;
        class_1799 stack = pContext.method_8041();
        class_2338 pos = pContext.method_8037();
        class_1937 level = pContext.method_8045();
        class_2680 state = level.method_8320(pos);
        String msgKey = getMessageTranslationKey();
        int maxDistance = getMaxDistanceFromSelection();

        if (player.method_5715() && stack.method_57826(AllDataComponents.CLICK_TO_LINK_DATA)) {
            if (level.method_8608())
                return class_1269.field_5812;
            player.method_7353(class_2561.method_43471("create." + msgKey + ".clear"), true);
            stack.method_57381(AllDataComponents.CLICK_TO_LINK_DATA);
            stack.method_57381(class_9334.field_49611);
            return class_1269.field_5812;
        }

        class_2960 placedDim = level.method_27983().method_29177();

        if (!stack.method_57826(AllDataComponents.CLICK_TO_LINK_DATA)) {
            if (!isValidTarget(level, pos)) {
                if (placeWhenInvalid()) {
                    class_1269 useOn = super.method_7884(pContext);
                    if (level.method_8608() || useOn == class_1269.field_5814)
                        return useOn;

                    class_1799 itemInHand = player.method_5998(pContext.method_20287());
                    if (!itemInHand.method_7960()) {
                        stack.method_57381(AllDataComponents.CLICK_TO_LINK_DATA);
                        stack.method_57381(class_9334.field_49611);
                    }
                    return useOn;
                }

                if (level.method_8608())
                    AllSoundEvents.DENY.playFrom(player);
                player.method_7353(class_2561.method_43471("create." + msgKey + ".invalid"), true);
                return class_1269.field_5814;
            }

            if (level.method_8608())
                return class_1269.field_5812;

            player.method_7353(class_2561.method_43471("create." + msgKey + ".set"), true);
            stack.method_57379(AllDataComponents.CLICK_TO_LINK_DATA, new ClickToLinkData(pos, placedDim));
            return class_1269.field_5812;
        }

        ClickToLinkData data = stack.method_58694(AllDataComponents.CLICK_TO_LINK_DATA);
        //noinspection DataFlowIssue
        class_2338 selectedPos = data.selectedPos();
        class_2960 selectedDim = data.selectedDim();
        class_2338 placedPos = pos.method_10079(pContext.method_8038(), state.method_45474() ? 0 : 1);

        if (maxDistance != -1 && (!selectedPos.method_19771(placedPos, maxDistance) || !selectedDim.equals(placedDim))) {
            player.method_7353(class_2561.method_43471("create." + msgKey + ".too_far").method_27692(class_124.field_1061), true);
            return class_1269.field_5814;
        }

        class_2487 beTag = new class_2487();
        beTag.method_67494("TargetOffset", class_2338.field_25064, selectedPos.method_10059(placedPos));
        beTag.method_67494("TargetDimension", class_2960.field_25139, selectedDim);
        stack.method_57379(class_9334.field_49611, class_11580.method_72535(((IBE<?>) method_7711()).getBlockEntityType(), beTag));

        class_1269 useOn = super.method_7884(pContext);
        if (level.method_8608() || useOn == class_1269.field_5814)
            return useOn;

        class_1799 itemInHand = player.method_5998(pContext.method_20287());
        if (!itemInHand.method_7960()) {
            stack.method_57381(AllDataComponents.CLICK_TO_LINK_DATA);
            stack.method_57381(class_9334.field_49611);
        }
        player.method_7353(class_2561.method_43471("create." + msgKey + ".success").method_27692(class_124.field_1060), true);
        return useOn;
    }

    public abstract int getMaxDistanceFromSelection();

    public abstract String getMessageTranslationKey();

    public boolean placeWhenInvalid() {
        return false;
    }

    public boolean isValidTarget(class_1936 level, class_2338 pos) {
        return true;
    }
}
